#if defined(HAS_LOCAL_NORMAL) || defined(HAS_VIEW_NORMAL) || defined(HAS_WORLD_NORMAL)
#    include "utils/getVertexNormal.glsl"
#endif

#ifdef DEPTH_CLAMP
#    include "utils/depthClampVert.glsl"
#endif

// ----------------------------------------
// ----------------------------------------

in vec3 aLocalVertex;

#if defined(HAS_COLOR)
in vec3 aColor;
#endif

#if defined(HAS_ROUGHNESS_METALNESS) || defined(HAS_MASK)
in vec3 aMaterial;
#endif

#ifdef HAS_UV
in vec2 aTexCoord;
#endif

// ----------------------------------------
// ----------------------------------------

#ifdef HAS_LOCAL_VERTEX
out vec3 vLocalVertex;
#endif

#ifdef HAS_WORLD_VERTEX
out vec3 vWorldVertex;
#endif

#ifdef HAS_VIEW_VERTEX
out vec3 vViewVertex;
#endif

// --------------------

#ifdef HAS_LOCAL_NORMAL
out vec3 vLocalNormal;
#endif

#ifdef HAS_WORLD_NORMAL
out vec3 vWorldNormal;
#endif

#ifdef HAS_VIEW_NORMAL
out vec3 vViewNormal;
#endif

// --------------------

#ifdef HAS_COLOR
out vec3 vBaseColor;
#endif

#if defined(HAS_ROUGHNESS_METALNESS)
out float vRoughness;
out float vMetalness;
#endif

#ifdef HAS_MASK
out float vMask;
#endif

#ifdef HAS_UV
out vec2 vTexCoord;
#endif

// ----------------------------------------
// ----------------------------------------

#ifdef HAS_VIEW_VERTEX
uniform mat4 uWorldViewMatrix;
#endif

#ifdef HAS_VIEW_NORMAL
uniform mat3 uWorldViewNormalMatrix;
#endif

#ifdef HAS_WORLD_VERTEX
uniform mat4 uWorldMatrix;
#endif

#ifdef HAS_WORLD_NORMAL
uniform mat3 uWorldNormalMatrix;
#endif

#ifndef NO_JITTER
uniform float uVertexJitter;
uniform vec3 uHalton;
#endif

uniform mat4 uWorldViewProjectionMatrix;
uniform vec2 uOutputSize;

// ----------------------------------------
// ----------------------------------------

void main(void) {
#ifdef HAS_COLOR
    vBaseColor = aColor.rgb;
#endif

#ifdef HAS_MASK
    vMask = aMaterial.b;
#endif

#ifdef HAS_ROUGHNESS_METALNESS
    vRoughness = aMaterial.r;
    vMetalness = aMaterial.g;
#endif

#ifdef HAS_UV
    vTexCoord = aTexCoord;
#endif

    // --------------------

#ifdef HAS_LOCAL_VERTEX
    vLocalVertex = aLocalVertex;
#endif

#ifdef HAS_WORLD_VERTEX
    vWorldVertex = vec3(uWorldMatrix * vec4(aLocalVertex, 1.0));
#endif

#ifdef HAS_VIEW_VERTEX
    vViewVertex = vec3(uWorldViewMatrix * vec4(aLocalVertex, 1.0));
#endif

    // --------------------

#ifdef HAS_LOCAL_NORMAL
    vLocalNormal = getVertexNormal();
#endif

#ifdef HAS_WORLD_NORMAL
    vWorldNormal = normalize(uWorldNormalMatrix * getVertexNormal());
#endif

#ifdef HAS_VIEW_NORMAL
    vViewNormal = normalize(uWorldViewNormalMatrix * getVertexNormal());
#endif

    // --------------------

    gl_Position = uWorldViewProjectionMatrix * vec4(aLocalVertex, 1.0);

#if !defined(NO_JITTER)
    gl_Position.xy += (uVertexJitter * gl_Position.w) * (uHalton.xy / uOutputSize.xy);
#endif

    // --------------------

#ifdef DEPTH_CLAMP
#    define INJECT_VERTEX depthClampVert();
#endif

#ifdef INJECT_VERTEX
    INJECT_VERTEX
#endif
}