#include "shading/environment.glsl"
#include "utils/colorSpace.glsl"

in vec3 vLocalVertex;

out vec3 oFragColor;

uniform int uBackgroundPreview;

// image 2d
uniform sampler2D uTexture0;
uniform vec2 uTexture0Size;
uniform vec2 uOutputSize;

// env
uniform float uBackgroundBlur;
uniform float uBackgroundExposure;
uniform float uEnvironmentExposure;

void main(void) {
    initUV();

    vec3 color = vec3(0.0);

    if (uBackgroundPreview == 1) {
        float phi = g_uv.x * 2.0 * M_PI;
        float theta = (1.0 - g_uv.y) * M_PI;

        float sinTheta = sin(theta);
        vec3 dir = normalize(vec3(-sinTheta * sin(phi), cos(theta), sinTheta * cos(phi)));

        // 5 is max exposure (gui)
        color = decodeLUV(texture(uTextureEnvironment, octaToUV(dir))) * uEnvironmentExposure;
        oFragColor = linearTosRGB(color);
        return;
    }

    vec3 dir = normalize(vLocalVertex);
    color = prefilterEnvMap(uBackgroundBlur, dir);

    // replace last lod with SPH
    float lod = uBackgroundBlur * MAX_LOD;
    float start = MAX_LOD - 1.0;
    float end = MAX_LOD;
    float r = clamp((lod - start) / (end - start), 0.0, 1.0);
    color = mix(color, computeDiffuseSPH(dir), r);

    color *= uEnvironmentExposure * uBackgroundExposure;

    // debug environment
    // vec2 uv = gl_FragCoord.xy / uTextureEnvironmentSize;
    // uv /= min(1.0, min(uOutputSize.x / uTextureEnvironmentSize.x, uOutputSize.y / uTextureEnvironmentSize.y));
    // color = decodeLUV(texture(uTextureEnvironment, vec2(uv.x, 1.0 - uv.y)));
    // if (uv.x > 1.0 || uv.y > 1.0) discard;

    oFragColor = color;
}
