#include "utils/getLuminance.glsl"
#include "utils/rand.glsl"

out vec4 oFragColor;

uniform sampler2D uTexture0;
uniform sampler2D uTexture1;
uniform sampler2D uTexture2;
uniform sampler2D uTexture3;
uniform sampler2D uTexture4;
uniform sampler2D uTexture5;

uniform vec2 uTexture1Size;
uniform vec2 uTexture2Size;
uniform vec2 uTexture3Size;
uniform vec2 uTexture4Size;
uniform vec2 uTexture5Size;

uniform float uBloomRadius;
uniform float uBloomIntensity;
uniform float uFrameJitter;

float getRadiusFactored(const float value, const float middle) {
    return mix(value, middle * 2.0 - value, uBloomRadius);
}

void main(void) {
    initUV();

    vec3 bloom = vec3(0.0);

    const float midVal = 0.6;

    float jitter = interleaveGradientNoise(gl_FragCoord.xy, uFrameJitter);

    bloom += texture(uTexture1, g_uv + 0.0 * jitter / uTexture1Size).rgb * getRadiusFactored(0.1, midVal);
    bloom += texture(uTexture2, g_uv + 0.0 * jitter / uTexture2Size).rgb * getRadiusFactored(0.9, midVal);
    bloom += texture(uTexture3, g_uv + 0.2 * jitter / uTexture3Size).rgb * getRadiusFactored(0.6, midVal);
    bloom += texture(uTexture4, g_uv + 0.5 * jitter / uTexture4Size).rgb * getRadiusFactored(0.3, midVal);
    bloom += texture(uTexture5, g_uv + 1.0 * jitter / uTexture5Size).rgb * getRadiusFactored(0.1, midVal);

    vec3 color = texture(uTexture0, g_uv).rgb;
    oFragColor = vec4(color + (bloom * uBloomIntensity), 1.0);
}