#include "composer/chromatic.glsl"
#include "composer/tonemap.glsl"
#include "utils/colorSpace.glsl"

out vec4 oFragColor;

uniform sampler2D uTexture0;
uniform sampler2D uTexture1;

#ifdef WRITE_ALPHA
uniform sampler2D uTextureDepth;
#endif

uniform vec3 uHalton;

void main(void) {
    initUV();

    vec3 color = vec3(0.0);

#ifdef HAS_CHROMATIC
    color = chromatic();
#else
    color = texture(uTexture0, g_uv).rgb;
#endif

#ifdef HAS_TONEMAP
    color = tonemap(color);
#else
    color = linearTosRGB(color);
#endif

    // accumulate
    if (uHalton.z < 1.0) {
        color = mix(texture(uTexture1, g_uv).rgb, color, uHalton.z);
    }

    oFragColor = vec4(color, 1.0);

#ifdef WRITE_ALPHA
    if (texture(uTextureDepth, g_uv).r == 1.0) oFragColor.a = 0.0;
#endif
}
