#include "composer/dofCommon.glsl"
#include "utils/rand.glsl"

out vec4 oFragColor;

uniform float uDofScale;
uniform float uQuality;
uniform float uFrameJitter;

uniform sampler2D uTexture0;
uniform vec2 uTexture0Size;

vec4 tmp;

#define DOF_TAP(id)                                                       \
    ofs = tapPoissons[id];                                                \
    ofs = g_uv + blurDist * vec2(dot(ofs, basis.xy), dot(ofs, basis.zw)); \
    tmp = texture(uTexture0, ofs);                                        \
    sumcol += vec4(tmp.rgb, 1.0) * abs(tmp.a * tmp.a * tmp.a);

vec4 dofBlur16(const in vec2 blurDist, const in float jitter) {
    float rnd = (M_PI * 2.0) * ditheringNoise(gl_FragCoord.xy, jitter);

    float cosa = cos(rnd);
    float sina = sin(rnd);
    vec4 basis = vec4(cosa, -sina, sina, cosa);

    vec4 sumcol = vec4(0.0);

    float total = 0.0;

    vec2 tapPoissons[17];
    // https://github.com/bartwronski/PoissonSamplingGenerator
    tapPoissons[0] = vec2(0.343280198355, 0.921191856785);
    tapPoissons[1] = vec2(-0.116859107262, 0.873056831344);
    tapPoissons[2] = vec2(-0.646421287853, 0.760425284853);
    tapPoissons[3] = vec2(-0.0437332058416, 0.47196719804);
    tapPoissons[4] = vec2(0.48240473891, 0.464894594736);
    tapPoissons[5] = vec2(-0.718897237297, 0.352191304304);
    tapPoissons[6] = vec2(-0.939061387886, -0.0387376698953);
    tapPoissons[7] = vec2(0.546569494557, -0.0169516524446);
    tapPoissons[8] = vec2(-0.182644095652, -0.0702667761349);
    tapPoissons[9] = vec2(-0.576191305396, -0.252368997099);
    tapPoissons[10] = vec2(0.921914032366, -0.366530187909);
    tapPoissons[11] = vec2(-0.0582053944545, -0.510617206206);
    tapPoissons[12] = vec2(-0.783101528934, -0.601411360136);
    tapPoissons[13] = vec2(0.430604354377, -0.729559481383);
    tapPoissons[14] = vec2(-0.350716524868, -0.809522919973);
    tapPoissons[15] = vec2(0.0600898835968, -0.952568587297);
    tapPoissons[16] = vec2(0.0, 0.0); // dummy because of intel off-by-one bug

    vec2 ofs;
    vec4 sampleTex;
    float cocWeight;

    DOF_TAP(0);
    DOF_TAP(1);
    DOF_TAP(2);
    DOF_TAP(3);
    DOF_TAP(4);
    DOF_TAP(5);
    DOF_TAP(6);
    DOF_TAP(7);
    DOF_TAP(8);
    DOF_TAP(9);
    DOF_TAP(10);
    DOF_TAP(11);
    DOF_TAP(12);
    DOF_TAP(13);
    DOF_TAP(14);
    DOF_TAP(15);

    return sumcol;
}

void main(void) {
    initUV();

    // if (skipDof()) {
    //     oFragColor = vec4(0.0);
    //     return;
    // }

    float factor = MAX_BLUR * uDofScale;

    vec2 blurDist = factor * abs(texture(uTexture0, g_uv).a) / uTexture0Size;

    float jitter = uFrameJitter * 3.0; // multiple by the number of max blur passes

    vec4 sumcol = dofBlur16(blurDist, jitter);                        // 16
    if (uQuality > 0.33) sumcol += dofBlur16(blurDist, jitter + 1.0); // 32
    if (uQuality > 0.66) sumcol += dofBlur16(blurDist, jitter + 2.0); // 48

    if (sumcol.a > 0.0) {
        oFragColor = vec4(sumcol.rgb / sumcol.a, 1.0);
    } else {
        oFragColor = vec4(0.0);
    }
}
