#include "composer/dofCommon.glsl"
#include "utils/readDepth.glsl"

out vec4 oFragColor;

uniform sampler2D uTexture0;         // 32
uniform sampler2D uTextureDofBlur;   // 32
uniform sampler2D uTextureDofDilate; // 32
uniform sampler2D uTextureDepth;     // 32

uniform vec2 uTextureDofDilateSize;
uniform vec2 uTextureDofBlurSize;

uniform float uDofScale;

void main(void) {
    initUV();

    vec3 color = texture(uTexture0, g_uv).rgb;

    // if (skipDof()) {
    //     oFragColor = vec4(color, 1.0);
    //     return;
    // }

    // half res color
    float ox = uDofScale / uTextureDofBlurSize.x;
    float oy = uDofScale / uTextureDofBlurSize.y;
    vec3 blur = texture(uTextureDofBlur, g_uv).rgb;
    blur += texture(uTextureDofBlur, g_uv + vec2(-ox, -oy)).rgb;
    blur += texture(uTextureDofBlur, g_uv + vec2(ox, -oy)).rgb;
    blur += texture(uTextureDofBlur, g_uv + vec2(-ox, oy)).rgb;
    blur += texture(uTextureDofBlur, g_uv + vec2(ox, oy)).rgb;
    blur /= 5.0;

    // merge full res and half res coc
    vec2 uvNearest = (floor(g_uv * uTextureDofDilateSize) + 0.5) / uTextureDofDilateSize;
    float cocDilate = texture(uTextureDofDilate, uvNearest).r;
    float cocSharp = depthToCoc(readDepthLinear(texture(uTextureDepth, g_uv).r));
    float coc = abs(min(cocDilate, cocSharp));

    // merge half res blurred color and full res pixels
    float cocLower = 0.05;
    float cocHigher = 0.3;

    oFragColor = vec4(mix(color, blur, smoothstep(cocLower, cocHigher, coc)), 1.0);
}
