#include "composer/dofCommon.glsl"

out vec4 oFragColor;

uniform float uDofScale;

uniform sampler2D uTexture0;
uniform vec2 uTexture0Size;

uniform sampler2D uTextureDepth;

void main(void) {
    initUV();

    // if (skipDof()) {
    //     oFragColor = vec4(0.0);
    //     return;
    // }

#ifdef DOF_DILATE_H
    vec2 offset = vec2(uDofScale / uTexture0Size.x, 0.0);
#else
    vec2 offset = vec2(0.0, uDofScale / uTexture0Size.y);
#endif

    // the weights and the formula can be tweaked, also we could add a bit of jitter or something
    vec2 ofs = MAX_BLUR * offset / 5.0;

    float coc0 = texture(uTexture0, g_uv).r;
    float coc1 = texture(uTexture0, g_uv - 5.0 * ofs).r;
    float coc2 = texture(uTexture0, g_uv - 4.0 * ofs).r;
    float coc3 = texture(uTexture0, g_uv - 3.0 * ofs).r;
    float coc4 = texture(uTexture0, g_uv - 2.0 * ofs).r;
    float coc5 = texture(uTexture0, g_uv - 1.0 * ofs).r;
    float coc6 = texture(uTexture0, g_uv + 1.0 * ofs).r;
    float coc7 = texture(uTexture0, g_uv + 2.0 * ofs).r;
    float coc8 = texture(uTexture0, g_uv + 3.0 * ofs).r;
    float coc9 = texture(uTexture0, g_uv + 4.0 * ofs).r;
    float coc10 = texture(uTexture0, g_uv + 5.0 * ofs).r;

    float coc = 0.0;

    if (texture(uTextureDepth, g_uv).r == 1.0) {
        // avoid "shrink" effect on the out of focus far part on the model edges
        // gaussian weights (sigma 10), (doesn't have to be gaussian though), but the sum must be 1.0
        // http://dev.theomader.com/gaussian-kernel-calculator/
        coc = abs(coc0) * 0.095474;
        coc += (abs(coc1) + abs(coc10)) * 0.084264;
        coc += (abs(coc2) + abs(coc9)) * 0.088139;
        coc += (abs(coc3) + abs(coc8)) * 0.091276;
        coc += (abs(coc4) + abs(coc7)) * 0.093585;
        coc += (abs(coc5) + abs(coc6)) * 0.094998;
        oFragColor = vec4(coc, 0.0, 0.0, 0.0);
        return;
    }

    // blur only near field (expand)
    coc = min(coc0, 0.0);
    coc = min(coc1 * 0.3, coc);
    coc = min(coc2 * 0.5, coc);
    coc = min(coc3 * 0.75, coc);
    coc = min(coc4 * 0.8, coc);
    coc = min(coc5 * 0.95, coc);
    coc = min(coc6 * 0.95, coc);
    coc = min(coc7 * 0.8, coc);
    coc = min(coc8 * 0.75, coc);
    coc = min(coc9 * 0.5, coc);
    coc = min(coc10 * 0.3, coc);
    if (abs(coc0) > abs(coc)) coc = coc0;

    oFragColor = vec4(coc, 0.0, 0.0, 0.0);
}