out vec4 oFragColor;

uniform sampler2D uTexture0;
uniform vec2 uTexture0Size;

uniform int uFxaa;

#define FXAA_G3D

// G3D version of FXAA
// green (y) as luma
// Quality 12
#ifdef FXAA_G3D
vec4 fxaa1() {
    float fxaaQualitySubpix = 0.75;
    float fxaaQualityEdgeThreshold = 0.125;
    float fxaaQualityEdgeThresholdMin = 0.0625;

    vec2 posM = g_uv;
    vec4 rgbyM = texture(uTexture0, posM);
    float lumaM = rgbyM.y;

    vec2 invSize = 1.0 / uTexture0Size;

    // Swizzler Offseter Util
    vec4 sw = vec4(-1.0, 1.0, 1.0, -1.0) * invSize.xxyy;

    float lumaS = texture(uTexture0, posM + vec2(0.0, sw.z)).y;
    float lumaE = texture(uTexture0, posM + vec2(sw.y, 0.0)).y;
    float lumaN = texture(uTexture0, posM + vec2(0.0, sw.w)).y;
    float lumaW = texture(uTexture0, posM + vec2(sw.x, 0.0)).y;

    float maxSM = max(lumaS, lumaM);
    float minSM = min(lumaS, lumaM);

    float maxESM = max(lumaE, maxSM);
    float minESM = min(lumaE, minSM);

    float maxWN = max(lumaN, lumaW);
    float minWN = min(lumaN, lumaW);

    float rangeMax = max(maxWN, maxESM);
    float rangeMin = min(minWN, minESM);

    float rangeMaxScaled = rangeMax * fxaaQualityEdgeThreshold;
    float range = rangeMax - rangeMin;
    float rangeMaxClamped = max(fxaaQualityEdgeThresholdMin, rangeMaxScaled);

    bool earlyExit = range < rangeMaxClamped;
    if (earlyExit) return rgbyM;

    float lumaNW = texture(uTexture0, posM + sw.xw).y;
    float lumaSE = texture(uTexture0, posM + sw.yz).y;
    float lumaNE = texture(uTexture0, posM + sw.yw).y;
    float lumaSW = texture(uTexture0, posM + sw.xy).y;

    float lumaNS = lumaN + lumaS;
    float lumaWE = lumaW + lumaE;
    float subpixRcpRange = 1.0 / range;
    float subpixNSWE = lumaNS + lumaWE;
    float edgeHorz1 = (-2.0 * lumaM) + lumaNS;
    float edgeVert1 = (-2.0 * lumaM) + lumaWE;

    float lumaNESE = lumaNE + lumaSE;
    float lumaNWNE = lumaNW + lumaNE;
    float edgeHorz2 = (-2.0 * lumaE) + lumaNESE;
    float edgeVert2 = (-2.0 * lumaN) + lumaNWNE;

    float lumaNWSW = lumaNW + lumaSW;
    float lumaSWSE = lumaSW + lumaSE;
    float edgeHorz4 = (abs(edgeHorz1) * 2.0) + abs(edgeHorz2);
    float edgeVert4 = (abs(edgeVert1) * 2.0) + abs(edgeVert2);
    float edgeHorz3 = (-2.0 * lumaW) + lumaNWSW;
    float edgeVert3 = (-2.0 * lumaS) + lumaSWSE;
    float edgeHorz = abs(edgeHorz3) + edgeHorz4;
    float edgeVert = abs(edgeVert3) + edgeVert4;

    float subpixNWSWNESE = lumaNWSW + lumaNESE;
    float lengthSign = invSize.x;
    bool horzSpan = edgeHorz >= edgeVert;
    float subpixA = subpixNSWE * 2.0 + subpixNWSWNESE;

    if (!horzSpan) lumaN = lumaW;
    if (!horzSpan) lumaS = lumaE;
    if (horzSpan) lengthSign = invSize.y;
    float subpixB = (subpixA * (1.0 / 12.0)) - lumaM;

    float gradientN = lumaN - lumaM;
    float gradientS = lumaS - lumaM;
    float lumaNN = lumaN + lumaM;
    float lumaSS = lumaS + lumaM;
    bool pairN = abs(gradientN) >= abs(gradientS);
    float gradient = max(abs(gradientN), abs(gradientS));
    if (pairN) lengthSign = -lengthSign;
    float subpixC = clamp(abs(subpixB) * subpixRcpRange, 0.0, 1.0);

    vec2 posB = posM.xy;
    vec2 offNP;
    offNP.x = (!horzSpan) ? 0.0 : invSize.x;
    offNP.y = (horzSpan) ? 0.0 : invSize.y;
    if (!horzSpan) posB.x += lengthSign * 0.5;
    if (horzSpan) posB.y += lengthSign * 0.5;

    vec2 posN;
    posN.x = posB.x - offNP.x;
    posN.y = posB.y - offNP.y;
    vec2 posP;
    posP.x = posB.x + offNP.x;
    posP.y = posB.y + offNP.y;
    float subpixD = ((-2.0) * subpixC) + 3.0;
    float lumaEndN = texture(uTexture0, posN).y;
    float subpixE = subpixC * subpixC;
    float lumaEndP = texture(uTexture0, posP).y;

    if (!pairN) lumaNN = lumaSS;
    float gradientScaled = gradient * 1.0 / 4.0;
    float lumaMM = lumaM - lumaNN * 0.5;
    float subpixF = subpixD * subpixE;
    bool lumaMLTZero = lumaMM < 0.0;

    lumaEndN -= lumaNN * 0.5;
    lumaEndP -= lumaNN * 0.5;
    bool doneN = abs(lumaEndN) >= gradientScaled;
    bool doneP = abs(lumaEndP) >= gradientScaled;
    if (!doneN) posN.x -= offNP.x * 1.5;
    if (!doneN) posN.y -= offNP.y * 1.5;
    bool doneNP = (!doneN) || (!doneP);
    if (!doneP) posP.x += offNP.x * 1.5;
    if (!doneP) posP.y += offNP.y * 1.5;

    if (doneNP) {
        if (!doneN) lumaEndN = texture(uTexture0, posN.xy).y;
        if (!doneP) lumaEndP = texture(uTexture0, posP.xy).y;
        if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
        if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
        doneN = abs(lumaEndN) >= gradientScaled;
        doneP = abs(lumaEndP) >= gradientScaled;
        if (!doneN) posN.x -= offNP.x * 2.0;
        if (!doneN) posN.y -= offNP.y * 2.0;
        doneNP = (!doneN) || (!doneP);
        if (!doneP) posP.x += offNP.x * 2.0;
        if (!doneP) posP.y += offNP.y * 2.0;

        if (doneNP) {
            if (!doneN) lumaEndN = texture(uTexture0, posN.xy).y;
            if (!doneP) lumaEndP = texture(uTexture0, posP.xy).y;
            if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
            if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
            doneN = abs(lumaEndN) >= gradientScaled;
            doneP = abs(lumaEndP) >= gradientScaled;
            if (!doneN) posN.x -= offNP.x * 4.0;
            if (!doneN) posN.y -= offNP.y * 4.0;
            doneNP = (!doneN) || (!doneP);
            if (!doneP) posP.x += offNP.x * 4.0;
            if (!doneP) posP.y += offNP.y * 4.0;

            if (doneNP) {
                if (!doneN) lumaEndN = texture(uTexture0, posN.xy).y;
                if (!doneP) lumaEndP = texture(uTexture0, posP.xy).y;
                if (!doneN) lumaEndN = lumaEndN - lumaNN * 0.5;
                if (!doneP) lumaEndP = lumaEndP - lumaNN * 0.5;
                doneN = abs(lumaEndN) >= gradientScaled;
                doneP = abs(lumaEndP) >= gradientScaled;
                if (!doneN) posN.x -= offNP.x * 12.0;
                if (!doneN) posN.y -= offNP.y * 12.0;
                doneNP = (!doneN) || (!doneP);
                if (!doneP) posP.x += offNP.x * 12.0;
                if (!doneP) posP.y += offNP.y * 12.0;
            }
        }
    }

    float dstN = posM.x - posN.x;
    float dstP = posP.x - posM.x;
    if (!horzSpan) dstN = posM.y - posN.y;
    if (!horzSpan) dstP = posP.y - posM.y;

    bool goodSpanN = (lumaEndN < 0.0) != lumaMLTZero;
    float spanLength = (dstP + dstN);
    bool goodSpanP = (lumaEndP < 0.0) != lumaMLTZero;
    float spanLengthRcp = 1.0 / spanLength;

    bool directionN = dstN < dstP;
    float dst = min(dstN, dstP);
    bool goodSpan = directionN ? goodSpanN : goodSpanP;
    float subpixG = subpixF * subpixF;
    float pixelOffset = (dst * (-spanLengthRcp)) + 0.5;
    float subpixH = subpixG * fxaaQualitySubpix;

    float pixelOffsetGood = goodSpan ? pixelOffset : 0.0;
    float pixelOffsetSubpix = max(pixelOffsetGood, subpixH);
    if (!horzSpan) posM.x += pixelOffsetSubpix * lengthSign;
    if (horzSpan) posM.y += pixelOffsetSubpix * lengthSign;

    return texture(uTexture0, posM);
}

#else

/**
Basic FXAA implementation based on the code on geeks3d.com with the
modification that the texture2DLod stuff was removed since it's
unsupported by WebGL.
--
From:
https://github.com/mitsuhiko/webgl-meincraft
Copyright (c) 2011 by Armin Ronacher.
Some rights reserved.
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.
    * The names of the contributors may not be used to endorse or
      promote products derived from this software without specific
      prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#    ifndef FXAA_REDUCE_MIN
#        define FXAA_REDUCE_MIN (1.0 / 128.0)
#    endif
#    ifndef FXAA_REDUCE_MUL
#        define FXAA_REDUCE_MUL (1.0 / 8.0)
#    endif
#    ifndef FXAA_SPAN_MAX
#        define FXAA_SPAN_MAX 8.0
#    endif

vec4 fxaa2() {
    vec2 invVP = 1.0 / uOutputSize;

    vec4 color;
    vec3 rgbNW = texture(uTexture0, g_uv + vec2(-1.0, -1.0) * invVP).xyz;
    vec3 rgbNE = texture(uTexture0, g_uv + vec2(1.0, -1.0) * invVP).xyz;
    vec3 rgbSW = texture(uTexture0, g_uv + vec2(-1.0, 1.0) * invVP).xyz;
    vec3 rgbSE = texture(uTexture0, g_uv + vec2(1.0, 1.0) * invVP).xyz;
    vec4 texColor = texture(uTexture0, g_uv);
    vec3 rgbM = texColor.xyz;
    vec3 luma = vec3(0.299, 0.587, 0.114);
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM = dot(rgbM, luma);
    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));

    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y = ((lumaNW + lumaSW) - (lumaNE + lumaSE));

    float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * FXAA_REDUCE_MUL), FXAA_REDUCE_MIN);

    float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);
    dir = min(vec2(FXAA_SPAN_MAX, FXAA_SPAN_MAX), max(vec2(-FXAA_SPAN_MAX, -FXAA_SPAN_MAX), dir * rcpDirMin)) * invVP;

    vec3 rgbA = 0.5 * (texture(uTexture0, g_uv + dir * (1.0 / 3.0 - 0.5)).xyz +
                       texture(uTexture0, g_uv + dir * (2.0 / 3.0 - 0.5)).xyz);
    vec3 rgbB =
        rgbA * 0.5 + 0.25 * (texture(uTexture0, g_uv + dir * -0.5).xyz + texture(uTexture0, g_uv + dir * 0.5).xyz);

    float lumaB = dot(rgbB, luma);
    if ((lumaB < lumaMin) || (lumaB > lumaMax))
        color = vec4(rgbA, texColor.a);
    else
        color = vec4(rgbB, texColor.a);
    return color;
}

#endif

vec4 fxaa() {
    if (uFxaa == 0) return texture(uTexture0, g_uv);
    return fxaa1();
}