out vec4 oFragColor;

uniform sampler2D uTexture0;
uniform vec2 uTexture0Size;
uniform vec2 uSsaoBlurDir;
uniform float uFrameJitter;

void main(void) {
    initUV();

    vec2 aoDepth = texture(uTexture0, g_uv).xy;

    // background
    if (aoDepth.g > 0.99) {
        oFragColor = vec4(1.0);
        return;
    }

    // no blur
    if (uFrameJitter > 0.5) {
        oFragColor = vec4(aoDepth.xy, 0.0, 0.0);
        return;
    }

    // {0.356642, 0.239400, 0.072410, 0.009869};
    // {0.398943, 0.241971, 0.053991, 0.004432, 0.000134};
    // {0.153170, 0.144893, 0.122649, 0.092902, 0.062970};
    // {0.111220, 0.107798, 0.098151, 0.083953, 0.067458, 0.050920, 0.036108};

    // https://github.com/PeterTh/gedosato/blob/master/pack/assets/dx9/SAO.fx#L504

    float gaussian[6]; // dummy because of intel off-by-one bug
    gaussian[0] = 0.153170;
    gaussian[1] = 0.144893 + 0.3;
    gaussian[2] = 0.122649 + 0.3;
    gaussian[3] = 0.092902 + 0.3;
    gaussian[4] = 0.062970 + 0.3;
    gaussian[5] = 0.0;

    float totalWeight = gaussian[0];
    float sum = aoDepth.x * totalWeight;

    vec2 axis = 0.5 * uSsaoBlurDir / uTexture0Size;
    float edgeSharpness = 2000.0;

    for (int id = -4; id <= 4; ++id) {
        if (id != 0) {
            vec4 sampleAoDepth = texture(uTexture0, g_uv + axis * float(id));
            float weight = gaussian[abs(id)] * max(0.0, 1.0 - edgeSharpness * abs(sampleAoDepth.y - aoDepth.y));

            sum += sampleAoDepth.x * weight;
            totalWeight += weight;
        }
    }

    oFragColor = vec4(sum / totalWeight, aoDepth.y, 0.0, 0.0);
}
