#include "utils/colorSpace.glsl"
#include "utils/getLuminance.glsl"

uniform float uToneExposure;
uniform float uToneBrightness;
uniform float uToneContrast;
uniform float uToneSaturation;
uniform int uToneMapping;

#define TONEMAP_ACES 1
#define TONEMAP_FILMIC 2
#define TONEMAP_REINHARD 3

vec3 tonemap(const in vec3 colorInput) {
    // exposure
    vec3 color = colorInput * uToneExposure;

    // saturation
    color = mix(vec3(getLuminance(color)), color, vec3(uToneSaturation));

    // float luminance = getLuminance(color * (1.0 + uToneBrightness));
    // color = mix(vec3(luminance), color * (1.0 + uToneBrightness), vec3(uToneSaturation));

    // contrast
    color = max(vec3(0.0), color * (1.0 + uToneContrast) - 0.5 * uToneContrast);

    if (uToneMapping == TONEMAP_FILMIC) {
        // https://github.com/TheRealMJP/BakingLab/blob/master/BakingLab/ToneMapping.hlsl#L58
        color = max(vec3(0.0), color - 0.004);
        color = (color * (6.2 * color + 0.5)) / (color * (6.2 * color + 1.7) + 0.06);

    } else if (uToneMapping == TONEMAP_ACES) {
        // Narkowicz 2015, "ACES Filmic Tone Mapping Curve"
        const float a = 2.51f;
        const float b = 0.03f;
        const float c = 2.43f;
        const float d = 0.59f;
        const float e = 0.14f;
        color = (color * (a * color + b)) / (color * (c * color + d) + e);
        color = linearTosRGB(color);

    } else if (uToneMapping == TONEMAP_REINHARD) {
        color /= 1.0 + getLuminance(color);
        color = linearTosRGB(color);

    } else {
        color = linearTosRGB(color);
    }

    return color;
}
