#include "utils/rand.glsl"

uniform float uFrameJitter;
uniform float uVignetteSize;
uniform float uVignetteHardness;

float vignette() {
    if (uVignetteSize == 0.0) {
        return 1.0;
    }

    vec2 lens = vec2(2.0 - uVignetteSize - uVignetteHardness, uVignetteHardness - uVignetteSize);

    // smoothesp is not realiable with edge0 == edge1
    lens.y = min(lens.y, lens.x - 1e-4);

    float jitter = interleaveGradientNoise(gl_FragCoord.xy, uFrameJitter);

    // jitter a bit the uv to remove the banding in some cases
    // (lens.x - lens.y) reduce flickering when the vignette is harder (hardness)
    // (lens.x + lens.y) reduce the flickering when the vignette has a strong radius
    jitter = (lens.x - lens.y) * (lens.x + lens.y) * 0.07 * (jitter - 0.5);
    return smoothstep(lens.x, lens.y, jitter + distance(g_uv, vec2(0.5)));
}
