#include "utils/checkDiscard.glsl"
#include "utils/depth.glsl"
#include "utils/encodeDecode.glsl"
#include "utils/encodeFragColor.glsl"
#include "utils/getNormal.glsl"
#include "utils/rand.glsl"

#if defined(SHADING_MATCAP)
#    include "shading/matcap.glsl"
#endif

in vec3 vBaseColor;
in float vRoughness;
in float vMetalness;

layout(location = 0) out vec4 oFrag0; // base color / roughness
layout(location = 1) out vec4 oFrag1; // metalness
layout(location = 2) out vec2 oFrag2; // normal

#ifdef DEPTH_SEPARATE
layout(location = 3) out float oFrag3;
#endif

uniform vec3 uBaseColor;
uniform float uRoughness;
uniform float uMetalness;

uniform int uMeshSelected;
uniform float uFrameJitter;

#ifdef SHADING_UV
uniform sampler2D uTexture0;
in vec2 vTexCoord;
#endif

vec3 getBaseColor() { return uBaseColor.x >= 0.0 ? uBaseColor : vBaseColor; }
float getMetalness() { return uMetalness >= 0.0 ? uMetalness : vMetalness; }
float getRoughness() { return uRoughness >= 0.0 ? uRoughness : vRoughness; }

void main(void) {
    g_frontSign = gl_FrontFacing ? 1.0 : -1.0;
    vec3 normal = getNormal(); // before discard
    checkDiscard();

    float metalness = getMetalness();
    float roughness = max(1.0 / 255.0, getRoughness());
    vec3 baseColor = getBaseColor();

#if defined(SHADING_MATCAP)
    baseColor *= matcap(normal);
#endif

#ifdef SHADING_UV
    baseColor *= sRGBToLinear(texture(uTexture0, vTexCoord).rgb);
#endif

    baseColor = encodeFragColor(baseColor).rgb;

    // if (uShowMask == 1) {
    //     metalness = mix(1.0, metalness, vMask);
    //     roughness = mix(1.0, roughness, vMask);
    // }

    // jitter
    float bias = interleaveGradientNoise(gl_FragCoord.xy, uFrameJitter);
    if (uFrameJitter > 0.5) {
        bias = (bias - 0.5) / 128.0;
        baseColor = saturate(baseColor + (0.5 - abs(baseColor - 0.5)) * bias);
    }

    oFrag0 = vec4(baseColor, roughness);
    oFrag1 = vec4(metalness, 1.0, 1.0, 1.0);
    oFrag2 = encodeNormal(normal);

    // ---------------------------

#if defined(DEPTH_SEPARATE) || defined(WRITE_FRAG_DEPTH_LINEAR)
#    ifdef DEPTH_SEPARATE
    oFrag3 = depthToLinear(gl_FragCoord.z);
#    endif

#    ifdef WRITE_FRAG_DEPTH_LINEAR
    gl_FragDepth = depthToLinear(gl_FragCoord.z);
#    endif
#endif
}
