in vec4 vTexCoord; // xy:texUv, zw:[0,1]
in vec4 vRounding; // xy:size, z:radius, w:rflags,
in vec4 vColor;

out vec4 oFragColor;

uniform sampler2D uTexture0;
uniform float uGuiFactor;

const float CORNER_BOTTOM_LEFT = 1.0;
const float CORNER_BOTTOM_RIGHT = 2.0;
const float CORNER_TOP_LEFT = 4.0;
const float CORNER_TOP_RIGHT = 8.0;

float getRounding() {
    vec2 size = vRounding.xy;
    float radius = vRounding.z;
    // radius = min(radius, min(size.x, size.y) * 0.5);

    vec2 nuv = vTexCoord.zw * size; // zw [0, 1]

    if (vRounding.a > 0.0) {
        // clang-format off

        // const int CORNER_NONE = 0;
        // const int CORNER_BOTTOM_LEFT = 1 << 0;
        // const int CORNER_BOTTOM_RIGHT = 1 << 1;
        // const int CORNER_TOP_LEFT = 1 << 2;
        // const int CORNER_TOP_RIGHT = 1 << 3;
        // int v = int(vRounding.a);
        // if ((v & CORNER_TOP_RIGHT) == 0 && nuv.x > radius && nuv.y > radius) nuv = vec2(min(nuv.x, radius), min(nuv.y, radius));
        // if ((v & CORNER_TOP_LEFT) == 0 && nuv.x < radius && nuv.y > radius) nuv = vec2(max(nuv.x, radius), min(nuv.y, radius));
        // if ((v & CORNER_BOTTOM_RIGHT) == 0 && nuv.x > radius && nuv.y < radius) nuv = vec2(min(nuv.x, radius), max(nuv.y, radius));
        // if ((v & CORNER_BOTTOM_LEFT) == 0 && nuv.x < radius && nuv.y < radius) nuv = vec2(max(nuv.x, radius), max(nuv.y, radius));

        // order is important (higher bit value first)
        float v = vRounding.a;
        if (v >= CORNER_TOP_RIGHT)    v -= CORNER_TOP_RIGHT;    else if (nuv.x > radius && nuv.y > radius) return 1.0;
        if (v >= CORNER_TOP_LEFT)     v -= CORNER_TOP_LEFT;     else if (nuv.x < radius && nuv.y > radius) return 1.0;
        if (v >= CORNER_BOTTOM_RIGHT) v -= CORNER_BOTTOM_RIGHT; else if (nuv.x > radius && nuv.y < radius) return 1.0;
        if (v >= CORNER_BOTTOM_LEFT)  v -= CORNER_BOTTOM_LEFT;  else if (nuv.x < radius && nuv.y < radius) return 1.0;
        // if (v < CORNER_BOTTOM_LEFT && nuv.x < radius && nuv.y < radius) return 1.0;
        // clang-format on
    }

    return 0.5 + radius - length(min(min(nuv, -nuv + size) - radius, 0.0));
}

void main(void) {
    vec4 tex = texture(uTexture0, vTexCoord.xy); // assumed premultiplied

    vec4 fragColor = vColor;
    fragColor.rgb = clamp(fragColor.rgb * uGuiFactor * tex.rgb, 0.0, 1.0);

    if (vRounding.z > 0.0 && vRounding.a > 0.0) {
        fragColor.a *= clamp(getRounding(), 0.0, 1.0);
    }

    oFragColor = vec4(fragColor.rgb * fragColor.a, fragColor.a * tex.a);
}
