vec3 g_diffuse;
vec3 g_specular;

#define FORWARD_VIEW_VERTEX vViewVertex

#include "utils/checkDiscard.glsl"
#include "utils/colorSpace.glsl"
#include "utils/encodeFragColor.glsl"
#include "utils/getNormal.glsl"
#include "utils/shading_pbr.glsl"
#include "utils/writeFragDepth.glsl"

in vec3 vViewVertex;
in vec3 vBaseColor;
in float vRoughness;
in float vMetalness;

out vec4 oFragColor;

uniform vec3 uBaseColor;
uniform float uRoughness;
uniform float uMetalness;

vec3 getBaseColor() { return uBaseColor.x >= 0.0 ? uBaseColor : vBaseColor; }
float getRoughness() { return uRoughness >= 0.0 ? uRoughness : vRoughness; }
float getMetalness() { return uMetalness >= 0.0 ? uMetalness : vMetalness; }

// ----------------------------------------------------------------
// ----------------------------------------------------------------

void main(void) {
    g_frontSign = gl_FrontFacing ? 1.0 : -1.0;
    vec3 normal = getNormal(); // before discard
    checkDiscard();

    initUV();

    g_diffuse = vec3(0.0);
    g_specular = vec3(0.0);

    vec3 baseColor = getBaseColor();

    shading_pbr(normal, baseColor, getRoughness(), getMetalness());

    oFragColor = encodeFragColor(g_diffuse + g_specular);

    writeFragDepth();
}
