#include "utils/colorSpace.glsl"

in vec3 vLocalVertex;

out vec4 oFragColor;

uniform int uConvertToSRGB;
uniform vec3 uBaseColor;
uniform float uMeshAlpha;

uniform vec3 uPieBase;
uniform float uPieAngle;

int pieAngle() {
    vec3 vn = normalize(vLocalVertex);
    float angle = acos(clamp(dot(vn, uPieBase), -1.0, 1.0));

    if (dot(vec3(0.0, 1.0, 0.0), cross(vn, uPieBase)) > 0.0) {
        angle = -angle;
    }

    float pieAngle = sign(uPieAngle) * mod(abs(uPieAngle), M_PI * 2.0);

    if (angle > 0.0) {
        if (pieAngle >= 0.0 && angle > pieAngle) return 0;
        if (pieAngle < -M_PI && angle < pieAngle + M_PI * 2.0) return 0;
        if (pieAngle < 0.0 && pieAngle > -M_PI) return 0;
    }

    if (angle < 0.0) {
        if (pieAngle <= 0.0 && angle < pieAngle) return 0;
        if (pieAngle > M_PI && angle > pieAngle - M_PI * 2.0) return 0;
        if (pieAngle > 0.0 && pieAngle < M_PI) return 0;
    }

    return 1;
}

void main(void) {
    if (length(vLocalVertex) > 1.0) {
        discard;
    }

    if (pieAngle() == 0) {
        discard;
    }

    oFragColor = vec4(uBaseColor * uMeshAlpha, uMeshAlpha);

    if (uConvertToSRGB == 1) {
        oFragColor = linearTosRGB(oFragColor);
    }
}
