#include "utils/colorSpace.glsl"
#include "utils/octahedronLod.glsl"

in vec3 vLocalVertex;

out vec4 oFragColor;

uniform sampler2D uTextureEnvironment;
uniform vec2 uTextureEnvironmentSize;

uniform sampler2D uTexturePrecomputeSample;
uniform vec2 uTexturePrecomputeSampleSize;

uniform vec2 uOutputSize;

#ifdef INPUT_LUV
#    define FETCH_IBL(uv) decodeLUV(texture(uTextureEnvironment, uv))
#else
#    define FETCH_IBL(uv) texture(uTextureEnvironment, uv).rgb
#endif

vec3 octaToDir(const in vec2 uv01) {
    vec2 uv = uv01 * 2.0 - 1.0;

    vec2 absUV = abs(uv);
    float z = 1.0 - absUV.x - absUV.y;
    if (z < 0.0) {
        uv.x = uv.x > 0.0 ? 1.0 - absUV.y : absUV.y - 1.0;
        uv.y = uv.y > 0.0 ? 1.0 - absUV.x : absUV.x - 1.0;
    }

    return normalize(vec3(uv.x, uv.y, z));
}

vec3 prefilterLod(const in float lod, const in vec3 dir) {
    vec2 uvBase = octaToUV(dir);
    vec3 f0 = FETCH_IBL(toUVMipmap(floor(lod), uvBase));
    vec3 f1 = FETCH_IBL(toUVMipmap(ceil(lod), uvBase));
    return f0 + (f1 - f0) * fract(lod);
    // return mix(f0, f1, fract(lod));
}

vec3 prefilterGGX(vec3 dir) {
    float totalWeight = 0.0;
    vec3 prefilterColor = vec3(0.0);

    float maxLod = log2(uTextureEnvironmentSize.x);

    vec3 axisZ = vec3(0.0, 0.0, 1.0);
    vec3 axisX = vec3(1.0, 0.0, 0.0);
    vec3 up = abs(dir.z) < 0.999 ? axisZ : axisX;
    vec3 tangentX = normalize(cross(up, dir));
    vec3 tangentY = normalize(cross(dir, tangentX));

    for (int i = 0; i < NUM_SAMPLES; ++i) {
        vec4 prec = texture(uTexturePrecomputeSample, float(i) / uTexturePrecomputeSampleSize);
        prec.xy = prec.xy * 2.0 - 1.0;

        vec3 sDir = normalize(tangentX * prec.x + tangentY * prec.y + dir * prec.z);
        prefilterColor += prefilterLod(prec.w * maxLod, sDir) * prec.z;
        totalWeight += prec.z;
    }
    return prefilterColor / totalWeight;
}

void main(void) {
    vec2 uv = (gl_FragCoord.xy - 0.5) / (uOutputSize - 1.0);
    uv.y = 1.0 - uv.y;

    vec3 dir = octaToDir(uv);
    vec3 color = prefilterGGX(dir);

    oFragColor = encodeLUV(color);
}
