vec3 g_diffuse;
vec3 g_specular;

#include "utils/colorSpace.glsl"
#include "utils/encodeDecode.glsl"

#ifdef SHADING_PBR_UV
#    include "utils/shading_pbr.glsl"
#endif

#ifdef HAS_CURVATURE
#    include "utils/curvature.glsl"
#endif

out vec3 oFragColor;

#ifdef HAS_SSAO
uniform sampler2D uTextureSsaoFinal;
#endif

uniform sampler2D uTextureGbuffer0;

#ifdef SHADING_PBR_UV
uniform sampler2D uTextureGbuffer1;
#endif

#if defined(SHADING_PBR_UV) || defined(HAS_CURVATURE)
uniform sampler2D uTextureGbuffer2;
#endif

#ifdef HAS_SSAO
uniform sampler2D uTextureSsaoFinal;
#endif

uniform vec2 uOutputSize;
uniform int uConvertToSRGB;

// ----------------------------------------------------------------
// ----------------------------------------------------------------

#ifdef SHADING_PBR_UV
void shading() {
    vec4 buf0 = texture(uTextureGbuffer0, g_uv);
    if (buf0.a == 0.0) {
        discard;
    }

    vec4 buf1 = texture(uTextureGbuffer1, g_uv);
    vec2 buf2 = texture(uTextureGbuffer2, g_uv).rg;

    vec3 baseColor = buf0.rgb;
    float roughness = buf0.a;
    float metalness = buf1.r;
    vec3 normal = decodeNormal(buf2.rg);

    shading_pbr(normal, baseColor, roughness, metalness);
}
#endif

// ----------------------------------------------------------------
// ----------------------------------------------------------------

#ifdef SHADING_MATCAP
void matcap() {
    vec3 buf0 = texture(uTextureGbuffer0, g_uv).rgb;

#    ifdef HAS_SSAO
    buf0.rgb *= texture(uTextureSsaoFinal, g_uv).r;
#    endif

    g_diffuse = buf0.rgb;
}
#endif

// ----------------------------------------------------------------
// ----------------------------------------------------------------

void main(void) {
    initUV();

    g_diffuse = vec3(0.0);
    g_specular = vec3(0.0);

#ifdef SHADING_PBR_UV
    shading();
#endif

#ifdef SHADING_MATCAP
    matcap();
#endif

    // ----------------------------------------

    vec3 finalColor = g_diffuse + g_specular;

#ifdef HAS_CURVATURE
    finalColor = applyCurvature(finalColor);
#endif

    if (uConvertToSRGB == 1) {
        oFragColor = linearTosRGB(finalColor);
    } else {
        oFragColor = finalColor;
    }
}
