#define LIGHT_DIRECTIONAL 0
#define LIGHT_SPOT 1
#define LIGHT_POINT 2

#REPEAT 3

#ifdef HAS_LIGHT_$ID$
uniform vec3 uLightColor$ID$;

#    if LIGHT_TYPE_$ID$ == LIGHT_DIRECTIONAL
uniform vec3 uLightViewDirection$ID$;

#    elif LIGHT_TYPE_$ID$ == LIGHT_SPOT
uniform vec3 uLightViewDirection$ID$;
uniform vec3 uLightViewPosition$ID$;
uniform float uLightSpotAngle$ID$;
uniform float uLightSpotSoftness$ID$;

#    elif LIGHT_TYPE_$ID$ == LIGHT_POINT
uniform vec3 uLightViewPosition$ID$;
#    endif

#endif

#ifdef HAS_SHADOW_$ID$
uniform sampler2D uTextureShadow$ID$;
uniform float uShadowNormalBias$ID$;
uniform mat4 uShadowProject$ID$;

#    if LIGHT_TYPE_$ID$ == LIGHT_SPOT
uniform vec2 uShadowNearFar$ID$;
#    endif

#endif

#REPEAT END

// ----------------------------------------------------

// #if defined(HAS_LIGHT_SPOT) || defined(HAS_LIGHT_POINT)
// float getDistanceAttenuation(float dist2, float invRad2) {
//     float att = dist2 * invRad2;
//     att = saturate(1.0 - att * att);
//     return (att * att) * (1.0 / (1.0 + dist2));
// }
// #endif

#ifdef HAS_LIGHT_SPOT
void precomputeSpot(const in vec3 normal, const in vec3 viewVertex, const in vec3 lightViewDirection,
                    const in vec3 lightViewPosition, const in float lightSpotAngle, const in float lightSpotSoftness,
                    out float attenuation, out vec3 eyeLightDir, out float dotNL) {

    eyeLightDir = lightViewPosition - viewVertex;
    float dist2 = dot(eyeLightDir, eyeLightDir);
    eyeLightDir = dist2 > 0.0 ? eyeLightDir / sqrt(dist2) : vec3(0.0, 1.0, 0.0);
    dotNL = dot(eyeLightDir, normal);

    // https://github.com/google/filament/blob/main/shaders/src/light_punctual.fs#L98
    // https://github.com/EpicGames/UnrealEngine/blob/df84cb430f38ad08ad831f31267d8702b2fefc3e/Engine/Shaders/Private/DynamicLightingCommon.ush#L65

    float cosCurAngle = dot(-eyeLightDir, lightViewDirection);
    // float spotFalloff = cosCurAngle * smoothstep(0.0, 1.0, (cosCurAngle - lightSpotAngle) / lightSpotSoftness);
    float spotFalloff = saturate((cosCurAngle - lightSpotAngle) / lightSpotSoftness);
    spotFalloff = spotFalloff * spotFalloff;

    // distance attenuation
    float distFalloff = 1.0 / (0.001 + dist2);

    attenuation = spotFalloff * distFalloff;
}
#endif

#ifdef HAS_LIGHT_POINT
void precomputePoint(const in vec3 normal, const in vec3 viewVertex, const in vec3 lightViewPosition,
                     out float attenuation, out vec3 eyeLightDir, out float dotNL) {

    eyeLightDir = lightViewPosition - viewVertex;
    float dist2 = dot(eyeLightDir, eyeLightDir);
    eyeLightDir = dist2 > 0.0 ? eyeLightDir / sqrt(dist2) : vec3(0.0, 1.0, 0.0);
    dotNL = dot(eyeLightDir, normal);

    attenuation = 1.0 / (0.001 + dist2);
}
#endif

#ifdef HAS_LIGHT_DIRECTIONAL
void precomputeDirectional(const in vec3 normal, const in vec3 lightViewDirection, out float attenuation,
                           out vec3 eyeLightDir, out float dotNL) {
    attenuation = 1.0;
    eyeLightDir = -lightViewDirection;
    dotNL = dot(eyeLightDir, normal);
}
#endif