#include "utils/colorSpace.glsl"

in vec3 vViewVertex;

uniform sampler2D uTextureMatcap;
uniform float uMatcapRotation;

vec2 uvMatcap(const in vec3 normal) {
    vec3 eyeVector = normalize(vViewVertex);
    vec3 nm_x = vec3(-eyeVector.z, 0.0, eyeVector.x);
    vec3 nm_y = cross(nm_x, eyeVector);
    return 0.5 + 0.5 * vec2(dot(normal.xz, nm_x.xz), dot(normal, nm_y));
}

// https://github.com/nidorx/matcaps#applying-matcaps
// vec2 uvMatcap(const vec3 normal) { return normal.xy * 0.5 + 0.5; }

// vec2 uvMatcap(const vec3 normal) {
//     vec3 ref = reflect(normalize(vViewVertex), normal);
//     // https://github.com/hughsk/matcap/blob/master/matcap.glsl
//     float m = 2.8284271247461903 * sqrt(ref.z + 1.0);
//     // https://www.clicktorelease.com/blog/creating-spherical-environment-mapping-shader/
//     float m = 2.0 * sqrt(pow(ref.x, 2.0) + pow(ref.y, 2.0) + pow(ref.z + 1.0, 2.0));
//     return ref.xy / m + 0.5;
// }

// https://gist.github.com/ayamflow/c06bc0c8a64f985dd431bd0ac5b557cd
vec2 rotateUV(const in vec2 uv, const in float rotation, const in float mid) {
    return vec2(cos(rotation) * (uv.x - mid) + sin(rotation) * (uv.y - mid) + mid,
                cos(rotation) * (uv.y - mid) - sin(rotation) * (uv.x - mid) + mid);
}

vec3 rotateY(const in vec3 v, const in float rotation) {
    float vcos = cos(rotation);
    float vsin = sin(rotation);

    return (vec3(v.x * vcos + v.z * vsin, v.y, -v.x * vsin + v.z * vcos));
}

vec3 matcap(const in vec3 normalIn) {
    vec3 normal = normalIn * 0.99;

    // float rot = uMatcapRotation * 4.0 * M_PI / 360.0;
    // float ext = 0.5 - abs(uMatcapRotation / 360.0 - 0.5);
    // normal.x += cos(rot) * ext * 1.0;
    // normal.y += sin(rot) * ext * 0.1;
    // normal = normalize(normal);

    normal = rotateY(normal, uMatcapRotation * (2.0 * M_PI / 360.0));

    vec2 uvMatcap = uvMatcap(normal);
    // uvMatcap = rotateUV(uvMatcap, uMatcapRotation * (2.0 * M_PI / 360.0), 0.5);
    uvMatcap.y *= -1.0;

    vec3 color = sRGBToLinear(texture(uTextureMatcap, uvMatcap).rgb);
    return color;
}
