#define LIN_SRGB(x) x < 0.0031308 ? x * 12.92 : 1.055 * pow(x, 1.0 / 2.4) - 0.055
#define SRGB_LIN(x) x < 0.04045 ? x *(1.0 / 12.92) : pow((x + 0.055) * (1.0 / 1.055), 2.4)

float linearTosRGB(const in float color) { return LIN_SRGB(color); }
vec3 linearTosRGB(const in vec3 color) { return vec3(LIN_SRGB(color.r), LIN_SRGB(color.g), LIN_SRGB(color.b)); }
vec4 linearTosRGB(const in vec4 color) {
    return vec4(LIN_SRGB(color.r), LIN_SRGB(color.g), LIN_SRGB(color.b), color.a);
}

float sRGBToLinear(const in float color) { return SRGB_LIN(color); }
vec3 sRGBToLinear(const in vec3 color) { return vec3(SRGB_LIN(color.r), SRGB_LIN(color.g), SRGB_LIN(color.b)); }
vec4 sRGBToLinear(const in vec4 color) {
    return vec4(SRGB_LIN(color.r), SRGB_LIN(color.g), SRGB_LIN(color.b), color.a);
}

// // should match glm
// #define RGBM_RANGE 6.0

// // http://graphicrants.blogspot.fr/2009/04/rgbm-color-encoding.html
// vec4 encodeRGBM(const in vec3 color) {
//     // if (RGBM_RANGE <= 0.0) return vec4(color, 1.0);
//     vec4 rgbm;
//     vec3 col = color / RGBM_RANGE;
//     rgbm.a = clamp(max(max(col.r, col.g), max(col.b, 1e-6)), 0.0, 1.0);
//     rgbm.a = ceil(rgbm.a * 255.0) / 255.0;
//     rgbm.rgb = col / rgbm.a;
//     return rgbm;
// }

// vec3 decodeRGBM(const in vec4 color) {
//     // if (RGBM_RANGE <= 0.0) return color.rgb;
//     return RGBM_RANGE * color.rgb * color.a;
// }

// https://mynameismjp.wordpress.com/2008/12/12/logluv-encoding-for-hdr/
const mat3 invLuvM = mat3(6.0013, -2.700, -1.7995, -1.332, 3.1029, -5.7720, 0.3007, -1.088, 5.6268);
vec3 decodeLUV(const in vec4 logLuv) {
    float Le = logLuv.z * 255.0 + logLuv.w;
    vec3 xp;
    xp.y = exp2((Le - 127.0) / 2.0);
    xp.z = xp.y / logLuv.y;
    xp.x = logLuv.x * xp.z;
    return max(invLuvM * xp, 0.0);
}

const mat3 luvM = mat3(0.2209, 0.3390, 0.4184, 0.1138, 0.6780, 0.7319, 0.0102, 0.1130, 0.2969);
vec4 encodeLUV(const vec3 rgb) {
    vec4 logLuv;
    vec3 xp = max(luvM * rgb, vec3(1e-6));

    logLuv.x = floor(255.0 * xp.x / xp.z);
    logLuv.y = floor(255.0 * xp.y / xp.z);

    float Le = 2.0 * log2(xp.y) + 127.0;
    logLuv.w = floor(255.0 * fract(Le));
    logLuv.z = (Le - logLuv.w / 255.0);

    return logLuv / 255.0;
}