#include "utils/encodeDecode.glsl"

uniform sampler2D uTextureGbuffer2;
uniform vec2 uTextureGbuffer2Size;

uniform vec4 uCurvatureBump;
uniform vec4 uCurvatureCavity;

float curvatureSoft(float curvature, float control) {
    if (curvature < 0.5 / control) {
        return curvature * (1.0 - curvature * control);
    }
    return 0.25 / control;
}

// vec3 curvatureSoft(float curv, vec3 col) {
//     return vec3(curvatureSoft(curv, col.r), curvatureSoft(curv, col.g), curvatureSoft(curv, col.b));
// }

vec3 applyCurvature(const in vec3 color) {
    vec3 offset = vec3(1.0 / uTextureGbuffer2Size, 0.0);

    float normalUp = decodeNormal(texture(uTextureGbuffer2, g_uv + offset.zy).rg).g;
    float normalDown = decodeNormal(texture(uTextureGbuffer2, g_uv - offset.zy).rg).g;
    float normalRight = decodeNormal(texture(uTextureGbuffer2, g_uv + offset.xz).rg).r;
    float normalLeft = decodeNormal(texture(uTextureGbuffer2, g_uv - offset.xz).rg).r;

    float normalDiff = normalUp - normalDown + normalRight - normalLeft;

    normalDiff += 0.0001;

    normalDiff = clamp(normalDiff, -1.f, 1.f);

    // if (normalDiff < 0.0) {
    //     return 1.0 + -2.0 * curvatureSoft(-normalDiff, uCurvatureCavity);
    // }

    // return 1.0 + 2.0 * curvatureSoft(normalDiff, uCurvatureBump);

    if (normalDiff > 0.0) {
        // normalDiff = 2.0 * curvatureSoft(abs(normalDiff), 0.0);
        return mix(color, uCurvatureBump.rgb, uCurvatureBump.a * normalDiff);
    }

    // normalDiff = 2.0 * curvatureSoft(abs(normalDiff), 0.0);
    return mix(color, uCurvatureCavity.rgb, uCurvatureCavity.a * -normalDiff);
}
