#include "utils/colorSpace.glsl"
#include "utils/frontSign.glsl"
#include "utils/getLuminance.glsl"

uniform int uMeshDarken;
uniform float uMeshAlpha;

uniform int uBackfaceColored;
uniform vec4 uBackfaceColor;

uniform int uOpaquePass;
uniform int uConvertToSRGB;
uniform int uShowMask;

uniform vec3 uSymmetryViewOrigin;
uniform vec3 uSymmetryViewNormal;
uniform int uSymmetryDisplay;

in float vMask;
in vec3 vViewVertex;

vec4 encodeFragColor(const in vec3 linearColor) {
    vec3 color = linearColor;

    // darkan (if unselected)
    if (uMeshDarken == 1) {
        color *= 0.2;
    }

    // mask
    if (uShowMask == 1) {
        color *= (0.05 + 0.95 * vMask);
    }

    // symmetry line
    if (uSymmetryDisplay == 1 && abs(dot(uSymmetryViewNormal, vViewVertex - uSymmetryViewOrigin)) < 0.005) {
        color = saturate(color + (getLuminance(color) < 0.5 ? 0.25 : -0.25));
    }

    // backface
    if (uBackfaceColored == 1 && g_frontSign < 0.0) {
        color = uBackfaceColor.rgb * mix(color, vec3(1.0), uBackfaceColor.a);
    }

    vec4 result = vec4(color, 1.0) * uMeshAlpha;

    if (uConvertToSRGB == 1) {
        result = linearTosRGB(result);
    }

    return result;
}
