#include "utils/colorSpace.glsl"
#include "utils/rand.glsl"

uniform float uFrameJitter;
uniform vec2 uTextureEnvironmentSize;
uniform sampler2D uTextureEnvironment;

#define MAX_LOD 4.0

vec2 toUVMipmap(const in float lod, const in vec2 uv) {
    float widthForLevel = uTextureEnvironmentSize.x / exp2(lod);

    float bias = 1.5;
    vec2 uvSpaceLocal = vec2(bias) + uv * (widthForLevel - bias * 2.0);

    // correct mipmap
    uvSpaceLocal.y += uTextureEnvironmentSize.y - widthForLevel * 2.0;

    // to uv space
    uvSpaceLocal /= uTextureEnvironmentSize;

    // flip
    uvSpaceLocal.y = 1.0 - uvSpaceLocal.y;
    return uvSpaceLocal;
}

#define FORCE_SIGN(x) x < 0.0 ? -1.0 : 1.0

vec2 octaToUV(const in vec3 dir) {
    vec3 signOct = vec3(FORCE_SIGN(dir.x), FORCE_SIGN(dir.y), FORCE_SIGN(dir.z));
    vec3 uvOct = dir / dot(dir, signOct);
    if (uvOct.z < 0.0) uvOct.xy = signOct.xy * (1.0 - abs(uvOct)).yx;
    return uvOct.xy * 0.5 + 0.5;
}

vec3 prefilterEnvMap(const in float rLinear, const in vec3 dir) {
    float lod = min(rLinear, 0.99) * MAX_LOD;

    float bias = ditheringNoise(gl_FragCoord.xy, uFrameJitter);

    // jitter ray (attenuate octahedral mapping pattern)
    vec2 uvBase = octaToUV(normalize(dir + mix(-2.0, 2.0, bias) / uTextureEnvironmentSize.x));

#ifdef ENV_FLOAT
    vec3 result = mix(texture(uTextureEnvironment, toUVMipmap(floor(lod), uvBase)).rgb,
                      texture(uTextureEnvironment, toUVMipmap(ceil(lod), uvBase)).rgb, fract(lod));
#else
    vec3 result = decodeLUV(mix(texture(uTextureEnvironment, toUVMipmap(floor(lod), uvBase)),
                                texture(uTextureEnvironment, toUVMipmap(ceil(lod), uvBase)), fract(lod)));
#endif

    return result;
}