// white vs interleaved vs blue noise
// https://blog.demofox.org/2017/10/31/animating-noise-for-integration-over-time/

// to test in a shadertoy
// https://www.shadertoy.com/view/lsdfD4

// https://www.shadertoy.com/view/4djSRW
// most combinations are possible : in[1,2,3] -> out[1,2,3]

#define INT_SCALE1 .1031
float pseudoRandom(const in vec2 fragCoord) {
    vec3 p3 = fract(vec3(fragCoord.xyx) * INT_SCALE1);
    p3 += dot(p3, p3.yzx + 19.19);
    return fract((p3.x + p3.y) * p3.z);
}

float randTri(const in float randIn) {
    // https://www.shadertoy.com/view/4t2SDh
    // Convert uniform distribution into triangle-shaped distribution.
    float orig = randIn * 2.0 - 1.0;
    float rnd = orig * inversesqrt(abs(orig));
    rnd = max(-1.0, rnd); // Nerf the NaN generated by 0*rsqrt(0). Thanks @FioraAeterna!
    rnd = rnd - sign(orig) + 0.5;
    return (rnd + 0.5) * 0.5;
}

// #define DEBUG_BLUE_NOISE

#ifdef DEBUG_BLUE_NOISE
uniform vec2 uOutputSize;
uniform vec3 uHalton;
uniform sampler2D uTextureBlueNoise;
#endif

// https://github.com/EpicGames/UnrealEngine/blob/release/Engine/Shaders/Private/Random.ush#L27
float interleaveGradientNoise(const in vec2 fragCoord, const in float frameMod) {
#ifdef DEBUG_BLUE_NOISE
    if (gl_FragCoord.x > uOutputSize.x * 0.5) {
        // return texture(uTextureBlueNoise, (gl_FragCoord.xy + frameMod * vec2(47.0, 17.0) * 0.695) / 64.0).r;
        return texture(uTextureBlueNoise, gl_FragCoord.xy / 64.0 + uHalton.xy).r;
    }
#endif

    vec3 magic = vec3(0.06711056, 0.00583715, 52.9829189);
    return fract(magic.z * fract(dot(fragCoord.xy + frameMod * vec2(47.0, 17.0) * 0.695, magic.xy)));
}

// https://github.com/EpicGames/UnrealEngine/blob/release/Engine/Shaders/Private/MaterialTemplate.ush#L1863
// we slighty change it by multiplying by 1.2 (to match with other noise), otherwise the range seems to be between [0 -
// 0.83]
float ditheringNoise(const in vec2 fragCoord, const in float frameMod) {
    // float fm = mod(frameMod, 2.0) == 0.0 ? 1.0 : -1.0;
    float fm = frameMod;
    float dither5 = fract((fragCoord.x + fragCoord.y * 2.0 - 1.5 + fm) / 5.0);
    float noise = fract(dot(vec2(171.0, 231.0) / 71.0, fragCoord.xy));
    return (dither5 * 5.0 + noise) * (1.2 / 6.0);
}

float pseudoRandom(void) { return pseudoRandom(gl_FragCoord.xy); }
float pseudoRandomTri(void) { return randTri(pseudoRandom(gl_FragCoord.xy)); }
float pseudoRandomTri(const in vec2 fragCoord) { return randTri(pseudoRandom(fragCoord)); }
float interleaveGradientNoise(void) { return interleaveGradientNoise(gl_FragCoord.xy, 0.0); }
float ditheringNoise(void) { return interleaveGradientNoise(gl_FragCoord.xy, 0.0); }
